@@ -73,11 +73,9 @@ App::App(int &argc, char **argv) : QApplication(argc, argv)
 #if defined(Q_OS_WIN32) || defined(Q_OS_MAC)
 	QIcon::setThemeName(APP_NAME);
 #endif // Q_OS_WIN32 || Q_OS_MAC
-#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
 	QIcon::setFallbackThemeName(APP_NAME);
-#endif // QT 5.12
 
-	_gui = new GUI();
+	_gui = new GUI(app->language());
 
 #ifdef Q_OS_ANDROID
 	connect(this, &App::applicationStateChanged, this, &App::appStateChanged);
