@@ -29,18 +29,25 @@ class MapData
 {
 public:
 	struct Poly {
-		Poly() : oneway(false) {}
+		Poly() : flags(0) {}
+
+		enum Flags {
+			OneWay = 1,
+			Invert = 2,
+			Direction = 4,
+			Dashed = 8
+		};
 
 		/* QPointF insted of Coordinates for performance reasons (no need to
 		   duplicate all the vectors for drawing). Note, that we do not want to
 		   ll2xy() the points in the MapData class as this can not be done in
 		   parallel. */
 		QVector<QPointF> points;
+		RectC boundingRect;
 		Label label;
 		Raster raster;
 		quint32 type;
-		RectC boundingRect;
-		bool oneway;
+		quint32 flags;
 
 		bool operator<(const Poly &other) const
 		  {return type > other.type;}
@@ -50,7 +57,6 @@ public:
 		Point() : id(0), flags(0) {}
 
 		enum Flags {
-			NoFlag = 0,
 			ClassLabel = 1,
 		};
 
@@ -72,7 +78,18 @@ public:
 		double yr;
 	};
 
-	MapData(const QString &fileName);
+	struct Polys {
+		QList<Poly> polygons;
+		QList<Poly> lines;
+	};
+
+	typedef QCache<const SubDiv*, Polys> PolyCache;
+	typedef QCache<const SubDiv*, QList<Point> > PointCache;
+	typedef QCache<const DEMTile*, Elevation> ElevationCache;
+
+	MapData(const QString &fileName, PolyCache &polyCache,
+	  PointCache &pointCache, ElevationCache &demCache, QMutex &lock,
+	  QMutex &demLock);
 	virtual ~MapData();
 
 	const QString &name() const {return _name;}
@@ -85,7 +102,7 @@ public:
 	void elevations(QFile *file, const RectC &rect, int bits,
 	  QList<Elevation> *elevations);
 
-	void load(qreal ratio);
+	void loadStyle(qreal ratio);
 	void clear();
 
 	bool hasDEM() const {return _hasDEM;}
@@ -114,15 +131,6 @@ protected:
 	QString _errorString;
 
 private:
-	struct Polys {
-		QList<Poly> polygons;
-		QList<Poly> lines;
-	};
-
-	typedef QCache<const SubDiv*, Polys> PolyCache;
-	typedef QCache<const SubDiv*, QList<Point> > PointCache;
-	typedef QCache<const DEMTile*, Elevation> ElevationCache;
-
 	struct PolyCTX
 	{
 		PolyCTX(QFile *file, const RectC &rect, const Zoom &zoom,
@@ -176,10 +184,10 @@ private:
 	static bool pointCb(VectorTile *tile, void *context);
 	static bool elevationCb(VectorTile *tile, void *context);
 
-	PolyCache _polyCache;
-	PointCache _pointCache;
-	ElevationCache _demCache;
-	QMutex _lock, _demLock;
+	PolyCache &_polyCache;
+	PointCache &_pointCache;
+	ElevationCache &_demCache;
+	QMutex &_lock, &_demLock;
 
 	friend class VectorTile;
 };
