@@ -156,7 +156,8 @@ Atlas::Atlas(const QString &fileName, bool TAR, const Projection &proj,
 				OziMap::CalibrationType type;
 				QString cf(calibrationFile(path, type));
 				if (cf.isNull()) {
-					qWarning("%s: no calibration file found", qPrintable(path));
+					qWarning("%s: no calibration file found",
+					  qUtf8Printable(path));
 					continue;
 				}
 				map = new OziMap(cf, type, proj, this);
@@ -165,8 +166,8 @@ Atlas::Atlas(const QString &fileName, bool TAR, const Projection &proj,
 			if (map->isValid())
 				_maps.append(map);
 			else {
-				qWarning("%s: %s",  qPrintable(map->path()),
-				  qPrintable(map->errorString()));
+				qWarning("%s: %s", qUtf8Printable(map->path()),
+				  qUtf8Printable(map->errorString()));
 				delete map;
 			}
 		}
@@ -322,10 +323,10 @@ void Atlas::draw(QPainter *painter, const QRectF &rect, int mapIndex,
 }
 
 void Atlas::load(const Projection &in, const Projection &out, qreal deviceRatio,
-  bool hidpi)
+  bool hidpi, int layer)
 {
 	for (int i = 0; i < _maps.count(); i++)
-		_maps.at(i)->load(in, out, deviceRatio, hidpi);
+		_maps.at(i)->load(in, out, deviceRatio, hidpi, layer);
 
 	computeZooms();
 	computeBounds();
